class Index:
    FG_COLOR = 0
    BG_COLOR = 1
    X = 0
    Y = 1
    RECT_X = 0
    RECT_Y = 1
    RECT_WIDTH = 2
    RECT_HEIGHT = 3
    SIZE_WIDTH = 0
    SIZE_HEIGHT = 1
    MARGIN_LEFT = 0
    MARGIN_TOP = 1
    MARGIN_RIGHT = 2
    MARGIN_BOTTOM = 3
